unit Kolibri;

interface

const
  clBlack   = $00000000;
  clNavy    = $00000080;
  clGreen   = $00008000;
  clTeal    = $00008080;
  clMaroon  = $00800000;
  clPurple  = $00800080;
  clOlive   = $00808000;
  clSilver  = $00C0C0C0;
  clGray    = $00808080;
  clBlue    = $000000FF;
  clLime    = $0000FF00;
  clAqua    = $0000FFFF;
  clRed     = $00FF0000;
  clFuchsia = $00FF00FF;
  clYellow  = $00FFFF00;
  clWhite   = $00FFFFFF;

const 
  M_WINDOW_TYPE1      = $0;
  M_WINDOW_NO_DRAW    = $01000000;
  M_WINDOW_TYPE2      = $02000000;
  M_WINDOW_SKIN       = $03000000;
  M_WINDOW_SKIN_FIXED = $04000000;
                            
  M_STYLE_CAPTION_GRADIENT = $80000000;
  M_STYLE_WORKAREA_COORD   = $40000000;
  M_STYLE_WORKAREA_NO_DRAW = $20000000;
  M_STYLE_HAS_CAPTION      = $10000000; 
  
  M_PIXEL_NEGATIVE = $01000000;
  
  M_FONT_SMALL = $0;
  M_FONT_LARGE = $10000000;
  
  M_BUTTON_DELETE    = $80000000;
  M_BUTTON_NO_DRAW   = $40000000;
  M_BUTTON_NO_BORDER = $20000000;
  
  M_GET_EVENT_REDRAW   = 1;
  M_GET_EVENT_KEYBOARD = 2;
  M_GET_EVENT_BUTTON   = 3;
  M_GET_EVENT_EXITAPP  = 4;
  M_GET_EVENT_MOUSE    = 6;
  M_GET_EVENT_IPC      = 7;
  M_GET_EVENT_NET      = 8;
  M_GET_EVENT_DEBUG    = 9;
  M_GET_EVENT_NO       = 0;
  
  M_BAR_GRADIENT = $40000000;
  
  M_BACKGROUND_SET_SIZE_WINDOW = 1;
  M_BACKGROUND_PUTPIXEL        = 2;
  M_BACKGROUND_REDRAW          = 3;
  M_BACKGROUND_TYPE            = 4;
  M_BACKGROUND_TYPE_SOLID      = 1;
  M_BACKGROUND_TYPE_SCALE      = 2;
  M_BACKGROUND_PUTIMAGE        = 5;
  
  M_SET_EVENT_REDRAW     = 1;
  M_SET_EVENT_KEYBOARD   = 2;
  M_SET_EVENT_BUTTON     = 4;
  M_SET_EVENT_EXITAPP    = 8;
  M_SET_EVENT_BACKGROUND = 16;
  M_SET_EVENT_MOUSE      = 32;
  M_SET_EVENT_IPC        = 64;
  M_SET_EVENT_NET        = 128;
  M_SET_EVENT_DEBUG      = 256;
  
  M_NUMBER_DEC = $0;
  M_NUMBER_HEX = $100;
  M_NUMBER_BIN = $200;
  
  fmClosed = $D7B0;
  fmInput  = $D7B1;
  fmOutput = $D7B2;
  fmInOut  = $D7B3;
  fmAppend = $D7B4;

type  
  TStructProcInfo = record
    Ticks: DWord;
    PosWin: word;
    NumSlot: word;
    Reserve1: word;
    Name: array [0..10] of char;
    Reserve3: byte;
    Addr: DWord;
    MemSize: DWord;
    PID: DWord;
    Left,Top: DWord;
    Width, Height: DWord;
    Slot: word;
    Reserve2: array [0..948] of byte;
  end;

type  
  TKolibri = object
  protected
    FTypeWindow, FTypeCaption: DWord;
  public
    constructor Init;
    procedure DefineWindow(Left, Top, Width, Height, ColorWorkarea, ColorCaption, ColorBorder: DWord);
    function  GetKey: DWord;
    procedure WriteText(Left, Top, Color: DWord; Font: byte; S: string);
    procedure WriteText(Left, Top, Color, ColorBackground: DWord; Font: Byte; S: string);
    function  WaitEvent: DWord;
    procedure BeginDraw;
    procedure EndDraw;
    procedure ExitProcess;
    function  GetButton: DWord;
    procedure WriteNumber(Left, Top, Color, State, Length, Number: DWord);
    procedure WriteNumber(Left, Top, Color, State, Length: DWord; Number: pointer);
    procedure SetEvents(Mask: DWord);
    procedure PutPixel(X, Y, Color: DWord);
    procedure Delay(Time: DWord);
    procedure PutImage(Image: pointer; X, Y, Width, Height: DWord);
    procedure DrawBar(X, Y, Width, Height, Color: DWord);
    procedure MouseCoordsScreen(var X, Y: DWord);
    procedure MouseCoordsWindow(var X, Y: DWord);
    function  MouseButtons: DWord;
    procedure DrawLine(StartX, StartY, EndX, EndY, Color: DWord);
    function  GetThreadInfo(Id: DWord; Data: pointer): DWord;
    function  CheckEvent: DWord;
    procedure DefineButton(Left, Top, width, Height, Id, Color: DWord);
    function  CheckEventTimeOut(TimeOut: DWord): DWord;
    procedure SetCaption(Caption: string);
    function  IPCSetMemory(Buffer: pointer; Size: DWord): DWord;
    function  IPCSendMessage(Id: DWord; Buffer: pointer; Size: DWord): DWord;
  end;

var
  API: TKolibri;

implementation

constructor TKolibri.Init;
begin
  FTypeWindow  := M_WINDOW_SKIN;
  FTypeCaption := M_STYLE_CAPTION_GRADIENT or M_STYLE_WORKAREA_COORD or M_STYLE_HAS_CAPTION;
end;

procedure TKolibri.DefineButton(Left, Top, Width, Height, Id, Color: DWord); assembler;
asm
  mov   eax, 8
  mov   ebx, dword ptr [Left]
  shl   ebx, 16
  add   ebx, dword ptr [Width]
  mov   ecx, dword ptr [Top]
  shl   ecx, 16
  add   ecx, dword ptr [Height]
  mov   edx, dword ptr [Id]
  mov   esi, dword ptr [Color]
  int   $40
end;

procedure TKolibri.DefineWindow(Left, Top, Width, Height, ColorWorkarea, ColorCaption, ColorBorder: DWord);
var
  TypeWindow, TypeCaption: DWord;
  Param: DWord;
begin
  TypeWindow  := FTypeWindow;
  TypeCaption := FTypeCaption;

  if (TypeWindow = M_WINDOW_SKIN) or 
     (TypeWindow = M_WINDOW_SKIN_FIXED)
    then Param := 0
    else Param := ColorBorder;
  
  asm
    xor   eax, eax
    mov   ebx, dword ptr [Left]
    shl   ebx, 16
    add   ebx, dword ptr [Width]
    mov   ecx, dword ptr [Top]
    shl   ecx, 16
    add   ecx, dword ptr [Height]
    mov   edx, dword ptr [ColorWorkarea]
    or    edx, dword ptr [TypeWindow]
    mov   esi, dword ptr [ColorCaption]
    or	  esi, dword ptr [TypeCaption]
    mov   edi, dword ptr [Param]
    int   $40
  end;
end;

function TKolibri.GetKey: DWord; assembler;
asm
  mov   eax, 2
  int   $40
  mov   dword ptr [Result], eax
end;

procedure TKolibri.WriteText(Left, Top, Color: DWord; Font: byte; S: string);
var 
  Size: DWord;
begin
  Size := length(S);
  asm
    mov   eax, 4
    mov   ebx, dword ptr [Left]
    shl   ebx, 16
    add   ebx, dword ptr [Top]
    mov   ecx, dword ptr [Color]
    or	  ecx, dword ptr [Font]
    mov   edx, dword ptr [S]
    inc   edx
    mov   esi, dword ptr [Size]
    int   $40
  end;
end;

procedure TKolibri.WriteText(Left, Top, Color, ColorBackground: DWord; Font: byte; S: string);
var 
  Size: DWord;
begin
  Size := length(S);
  asm
    mov   eax, 4
    mov   ebx, dword ptr [Left]
    shl   ebx, 16
    add   ebx, dword ptr [Top]
    mov   edi, dword ptr [ColorBackground]
    mov   ecx, dword ptr [Color]
    or	  ecx, dword ptr [Font]
    add	  ecx, $40000000
    mov   edx, dword ptr [S]
    inc   edx
    mov   esi, dword ptr [Size]
    int   $40
  end;
end;

function TKolibri.WaitEvent: DWord; assembler;
asm
  mov   eax, 10
  int   $40
  mov   dword ptr [Result], eax
end;

procedure TKolibri.BeginDraw; assembler;
asm
  mov   eax, 12
  mov   ebx, 1
  int   $40
end;

procedure TKolibri.EndDraw; assembler;
asm
  mov   eax, 12
  mov   ebx, 2
  int   $40
end;

procedure TKolibri.ExitProcess; assembler;
asm
  mov   eax, -1
  int   $40
end;

function TKolibri.GetButton: DWord; assembler;
asm
  mov   eax, 17
  int   $40
  mov   dword ptr [Result], eax
end;

procedure TKolibri.WriteNumber(Left, Top, Color, State, Length, Number: DWord);
var
  Param: DWord;
begin
  Param := Length * 65536 + State;
  asm
    mov  eax, 47
    mov  edx, dword ptr [Left]
    shl  edx, 16
    add  edx, dword ptr [Top]
    mov  ecx, dword ptr [Number]
    mov  esi, dword ptr [Color]
    mov  ebx, dword ptr [Param]
    mov  bl, 0
    int  $40
  end;
end;

procedure TKolibri.WriteNumber(Left, Top, Color, State, Length: DWord; Number: pointer);
var 
  Param: DWord;
begin
  Param := Length * 65536 + State;
  asm
    mov  eax, 47
    mov  edx, dword ptr [Left]
    shl  edx, 16
    add  edx, dword ptr [Top]
    mov  ecx, dword ptr [Number]
    mov  esi, dword ptr [Color]
    mov  ebx, dword ptr [Param]
    mov  bl, 1
    int  $40
  end;
end;

procedure TKolibri.SetEvents(Mask: DWord); assembler;
asm
  mov   eax, 40
  mov   ebx, dword ptr [Mask]
  int   $40
end;

procedure TKolibri.PutPixel(X, Y, Color: DWord); assembler;
asm
  mov   ebx, dword ptr [X]
  mov   ecx, dword ptr [Y]
  mov   edx, dword ptr [Color]
  mov   eax, 1
  int   $40
end;

procedure TKolibri.Delay(Time: DWord); assembler;
asm
  mov   eax, 5
  mov   ebx, dword ptr [Time]
  int   $40
end;

procedure TKolibri.PutImage(Image: pointer; X, Y, Width, Height: DWord); assembler;
asm
  mov   eax, 7
  mov   ebx, dword ptr [Image]
  mov   edx, dword ptr [X]
  shl   edx, 16
  add   edx, dword ptr [Y]
  mov   ecx, dword ptr [Width]
  shl   ecx, 16
  add   ecx, dword ptr [Height]
  int   $40
end;

procedure TKolibri.DrawBar(X, Y, Width, Height, Color: DWord); assembler;
asm
  mov   eax, 13
  mov   ebx, dword ptr [X]
  shl   ebx, 16
  add   ebx, dword ptr [Width]
  mov   ecx, dword ptr [Y]
  shl   ecx, 16
  add   ecx, dword ptr [Height]
  mov   edx, dword ptr [Color]
  int   $40
end;

procedure TKolibri.MouseCoordsScreen(var X, Y: DWord); Assembler;
asm
  mov   eax, 37
  xor   ebx, ebx
  int   $40
  mov   ecx, eax
  and   ecx, $ffff
  mov   ebx, dword ptr [y]
  mov   dword ptr [ebx],ecx
  shr   eax, 16
  mov   ecx, eax
  and   ecx, $ffff
  mov   ebx, dword ptr [x]
  mov   dword ptr [ebx], ecx
end;

procedure TKolibri.MouseCoordsWindow(var X, Y: DWord); assembler;
asm
  mov   eax, 37
  mov   ebx, 1
  int   $40
  mov   ecx, eax
  and   ecx, $ffff
  mov   ebx, dword ptr [Y]
  mov   dword ptr [ebx], ecx
  shr   eax, 16
  mov   ecx, eax
  and   ecx, $ffff
  mov   ebx, dword ptr [X]
  mov   dword ptr [ebx], ecx
end;

function TKolibri.MouseButtons: DWord; assembler;
asm
  mov   eax, 37
  mov   ebx, 2
  int   $40
  mov   dword ptr [Result], eax
end;

procedure TKolibri.DrawLine(StartX, StartY, EndX, EndY, Color: DWord); assembler;
asm
  mov   eax, 38
  mov   ebx, dword ptr [StartX]
  shl   ebx, 16
  add   ebx, dword ptr [EndX]
  mov   ecx, dword ptr [StartX]
  shl   ecx, 16
  add   ecx, dword ptr [EndX]
  mov   edx, dword ptr [Color]
  int   $40
end;

function TKolibri.GetThreadInfo(Id: DWord; Data: pointer): DWord; assembler;
asm
  mov   eax, 9
  mov   ebx, dword ptr [Data]
  mov   ecx, dword ptr [Id]
  int   $40
  mov   dword ptr [Result], eax
end;

function TKolibri.CheckEvent: DWord; assembler;
asm
  mov   eax, 11
  int   $40
  mov   dword ptr [Result], eax
end;

function TKolibri.CheckEventTimeOut(TimeOut: DWord): DWord; assembler;
asm
  mov   eax, 23
  mov   ebx, dword ptr [TimeOut]
  int   $40
  mov   dword ptr [Result], eax
end;

procedure TKolibri.SetCaption(Caption: string);
begin
  if length(Caption) >= 255
    then Caption[255] := #0
    else Caption[length(Caption) + 1] := #0;

  asm
    mov   eax, 71
    mov   ebx, 1
    mov   ecx, dword ptr [Caption]
    inc   ecx
    int   $40
  end;
end;

function TKolibri.IPCSetMemory(Buffer: pointer; Size: DWord): DWord; assembler;
asm
  mov   eax, 60
  mov   ebx, 1
  mov   ecx, dword ptr [Buffer]
  mov   edx, dword ptr [Size]
  int   $40
  mov   dword ptr [Result], eax
end;

function TKolibri.IPCSendMessage(Id: DWord; Buffer: pointer; Size: DWord): DWord; assembler;
asm
  mov   eax, 60
  mov   ebx, 2
  mov   ecx, dword ptr [Id]
  mov   edx, dword ptr [Buffer]
  mov   esi, dword ptr [Size]
  int   $40
  mov   dword ptr [Result], eax
end;

begin
  API.Init;
end.
