program Test;

uses 
  LoadPE, Kolibri;

type
  PApplication = ^TApplication;
  TApplication = object
  protected
    FLeft, FTop, FWidth, FHeight: DWord;
    FCaption: string;
    procedure Draw; virtual;
    procedure Process(Event: DWord); virtual;
    procedure Idle; virtual;
  public
    constructor Init(Left, Top, Width, Height: DWord; Caption: string);
    destructor  Done; virtual;
    procedure Run; virtual;
  end;

constructor TApplication.Init(Left, Top, Width, Height: DWord; Caption: string);
begin
  FLeft    := Left;
  FTop     := Top;
  FWidth   := Width;
  FHeight  := Height;
  FCaption := Caption;
end;

destructor TApplication.Done;
begin
  Kolibri.API.ExitProcess
end;

procedure TApplication.Run;
var
  Event: DWord;
begin
  Draw;
  Kolibri.API.SetEvents(M_SET_EVENT_REDRAW or M_SET_EVENT_KEYBOARD or M_SET_EVENT_BUTTON or M_SET_EVENT_MOUSE);
  repeat
    Event := Kolibri.API.CheckEventTimeOut(50);
    case Event Of
      M_GET_EVENT_NO: Idle;
      else Process(Event);
    end;
  until false;
end;

procedure TApplication.Process(Event: DWord);
begin
  case Event Of
    M_GET_EVENT_REDRAW: begin
      Draw;
    end;
    M_GET_EVENT_KEYBOARD: begin
      Kolibri.API.GetKey;
    end;
    M_GET_EVENT_BUTTON: begin
      Kolibri.API.GetButton;
      Kolibri.API.ExitProcess;
    end;
  end;
end;

procedure TApplication.Idle;
begin
  { empty }
end;

procedure TApplication.Draw;
begin
  Kolibri.API.BeginDraw;
  Kolibri.API.DefineWindow(FLeft, FTop, FWidth, FHeight, 0, 0, 0);
  Kolibri.API.SetCaption(FCaption);
  Kolibri.API.WriteText(6, 30, $FFFFFF, M_FONT_SMALL,'Hello, World!');
  Kolibri.API.EndDraw;
end;

var 
  TestApp: PApplication;

begin
  TestApp := new(PApplication, Init(10, 10, 300, 150,'Pascal Pro Test'));
  TestApp^.Run;
  Dispose(TestApp, Done());
end.
